//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Thread
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   03.06.2004
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2005-09-13 11:03:31 +0200 (Di, 13 Sep 2005) $
 *   $Revision: 1.2 $
 */
//-----------------------------------------------------------------------------

//=============================================================================
//     includes
//=============================================================================

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Tools/Trace.h>
// #include <Ax/Tools/TraceOn.h>
#include "Ax/VDR/Thread.h"

//----- locals ----------------------------------------------------------------

//=============================================================================
//     namespaces
//=============================================================================
namespace Ax
{
namespace VDR
{

//=============================================================================
//     class Thread
//=============================================================================

//-------------------------------------------------------------------------
//     Thread::Thread()
//-------------------------------------------------------------------------
Thread::Thread(const std::string &theDescription)
      : PARENT         (theDescription.c_str())
      , myName         (theDescription)
      , myStopRequested(false)
{
  wsdebug(("> Thread::Thread()"));
  wsdebug(("< Thread::Thread()"));
} // Thread::Thread()

//-------------------------------------------------------------------------
//     Thread::~Thread()
//-------------------------------------------------------------------------
Thread::~Thread()
{
  wsdebug(("> Thread::~Thread() Active: %d", isActive()));
  if (isActive())
  {
    stop();
  } // if
  wsdebug(("< Thread::~Thread() Active: %d", isActive()));
} // Thread::~Thread()

//-------------------------------------------------------------------------
//     Thread::getName()
//-------------------------------------------------------------------------
std::string Thread::getName() const
{
  std::string aName;
  {
    cMutexLock aLock(&myNameMutex);
    aName = myName;
  }
  return aName;
} // Thread::getName()

//-------------------------------------------------------------------------
//     Thread::start()
//-------------------------------------------------------------------------
bool Thread::start()
{
  bool fRunning = true;

  wsdebug(("> Thread::start() Active: %d", isActive()));
  if (!isActive())
  {
    myStopRequested = false;
    wsinfo(("%s: starting thread...", getName().c_str()));
    fRunning = PARENT::Start();
    wsinfo(("%s: starting thread...%s", getName().c_str(), fRunning ? "OK" : "FAILED"));
  } // if
  wsdebug(("< Thread::start() Active: %d", isActive()));
  return fRunning;
} // Thread::start()

//-------------------------------------------------------------------------
//     Thread::stop()
//-------------------------------------------------------------------------
void Thread::stop(int theWaitSeconds)
{
  wsdebug(("> Thread::stop() Active: %d  WaitSecs: %d", isActive(), theWaitSeconds));
  if (isActive())
  {
    if (!stopRequested()) setStopRequested();

    wsinfo(("%s: cancelling and killing in %d secs...", getName().c_str(), theWaitSeconds));
    Cancel(theWaitSeconds);

    wsinfo(("%s: cancelling and killing in %d secs...done", getName().c_str(), theWaitSeconds));
  } // if
  wsdebug(("< Thread::stop() Active: %d  WaitSecs: %d", isActive(), theWaitSeconds));
} // Thread::stop()

//-------------------------------------------------------------------------
//     Thread::signalEvent()
//-------------------------------------------------------------------------
void Thread::signalEvent()
{
  if (isActive())
  {
    wsdebug(("  Thread::signalEvent() waking up thread..."));
    myWaitCondition.Signal();
  }
  else
  {
    wsdebug(("  Thread::signalEvent() Thread not active -> nothing to do!"));
  } // if
} // Thread::signalEvent()

//-------------------------------------------------------------------------
//     Thread::setStopRequested()
//-------------------------------------------------------------------------
void Thread::setStopRequested()
{
  if (isActive())
  {
    wsinfo(("%s: setting flag to stop the thread...", getName().c_str()));
    {
      LOCK_THREAD;
      myStopRequested = true;
    }
    wsinfo(("%s: setting flag to stop the thread...done", getName().c_str()));

    wsinfo(("%s: signalling an event to the thread...", getName().c_str()));
    signalEvent();
    wsinfo(("%s: signalling an event to the thread...done", getName().c_str()));
  } // if
} // Thread::setStopRequested()

//-------------------------------------------------------------------------
//     Thread::stopRequested()
//-------------------------------------------------------------------------
bool Thread::stopRequested()
{
  bool fStopRequested = false;

  {
    LOCK_THREAD;
    fStopRequested = myStopRequested;
  }

  return fStopRequested;
} // Thread::stopRequested()

//-------------------------------------------------------------------------
//     Thread::waitEvent()
//-------------------------------------------------------------------------
bool Thread::waitEvent(int theTimeOutMS, bool *fTimeOut)
{
  bool fContinue = Running() && !stopRequested();

  if (!fContinue)
  {
    wsinfo(("%s: waitEvent: Running: %d  StopRequested: %d  fContinue: %d", getName().c_str(), Running(), stopRequested(), fContinue));
  }
  else
  {
    wsdebug(("  Thread::waitEvent() waiting for signal for %d ms...", theTimeOutMS));
    bool fSignal = myWaitCondition.Wait(theTimeOutMS);
    if (!fSignal && fTimeOut)
    {
      *fTimeOut = true;
    } // if
    fContinue = Running() && !stopRequested();
    if (!fContinue)
    {
      wsinfo(("%s: waitEvent: Running: %d  StopRequested: %d  fContinue: %d  Signal: %d"
            , getName().c_str(), Running(), stopRequested(), fContinue, fSignal
            ));
    } // if
  } // if

  return fContinue;
} // Thread::waitEvent()

}; // namespace VDR
}; // namespace Ax
